<?php

class modelLanguages extends cmsModel {
	
	public $table_prefix = 'con_';
	public $supports = array('caption', 'string', 'html', 'text');
	
	const BASE_URL = 'https://translate.yandex.net/api/v1.5/tr.json/';
    const MESSAGE_UNKNOWN_ERROR = 'Unknown error';
    const MESSAGE_JSON_ERROR = 'JSON parse error';
    const MESSAGE_INVALID_RESPONSE = 'Invalid response from service';

	
	public function createContentsTables($translate_fields){
        $model = cmsCore::getModel('content');
		$ctypes = $model->getContentTypesNames();
		$config = cmsConfig::getInstance();

		if($ctypes)
		foreach($ctypes as $key=>$ctype){
			
			$sql = "CREATE TABLE IF NOT EXISTS `{#}languages_con_".$ctype."` (
					  `id` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
					  `item_id` int(11) UNSIGNED DEFAULT NULL,				  
					  `ctype` varchar(32) NOT NULL DEFAULT '',
					  `lang` varchar(5) NOT NULL DEFAULT '',";						
			
			$fields = $model->getContentFields($ctype);
			foreach($fields as $field) {
				if(in_array($field['type'], $translate_fields)) $sql .= "`".$field['name']."` text DEFAULT NULL,";
			}
	
			$sql .= "PRIMARY KEY (`id`)) ENGINE=".$config->db_engine." DEFAULT CHARSET=utf8";			
			$result = $this->db->query($sql);				
		}

	}
	
	public function getCatTitle($ctype, $item_id, $lang){
        return $this->filterEqual('item_id', $item_id)->filterEqual('ctype', $ctype)->filterEqual('lang', $lang)->getFieldFiltered('languages_cats', 'title');
    }
	
	
	public function createFields($table_name, $structure){
        return  $this->db->createTable($table_name, $structure);
    }	
	
	
	/*===================================================*/
	public function getCtypes(){
        return $this->get('content_types');
    }	
	
	public function getTranslateFields($ctype){
        return  $this->filterEqual('ctype', $ctype)->get('languages_fields');
    }	
	
	public function getTranslateField($ctype, $id, $lang){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $id)->filterEqual('lang', $lang)->get('languages_fields');
    }	
	
	public function getTranslateFieldTitle($ctype, $id, $lang){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_fields', 'title');;
    }	
	
	public function getTranslateDatasetTitle($ctype, $id, $lang){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_datasets', 'title');;
    }	
	
	public function getTranslateTitle($ctype, $id, $lang, $type){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$type, 'title');;
    }	
	
	public function getTranslateMenu($ctype, $id, $lang, $type){		
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$type, 'title');
    }
	
	public function getTranslateMenuLinks($ctype, $id, $lang, $type){		
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$type, 'links');
    }	
	
	public function getTranslateItem($ctype, $id, $lang){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $id)->filterEqual('lang', $lang)->get('languages_con_'.$ctype);
    }	
	
	public function getTranslateCtype($id, $lang){
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->get('languages_ctypes');
    }	
	
	public function getDatasetTitle($item_id, $lang, $ctype){
        return $this->filterEqual('item_id', $item_id)->filterEqual('lang', $lang)->filterEqual('ctype', $ctype)->getFieldFiltered('languages_datasets', 'title');
    }
	
	public function getTranslateTag($lang, $tag_id){
        return $this->filterEqual('item_id', $tag_id)->filterEqual('lang', $lang)->getFieldFiltered('languages_tags', 'title');
    }	
				
	public function getTranslateTagByName($lang, $name){
        return $this->filterEqual('ctype', $name)->filterEqual('lang', $lang)->getFieldFiltered('languages_tags', 'title');
    }
	
	public function getTranslatePropTitle($ctype, $lang, $id){
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_con_'.$ctype.'_props', 'title');
    }	
	
	public function getTranslatePropFieldset($ctype, $lang, $id){
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_con_'.$ctype.'_props', 'fieldset');
    }		
	
	public function getTranslatePropVals($ctype, $lang, $id){
        return $this->filterEqual('item_id', $id)->filterEqual('lang', $lang)->getFieldFiltered('languages_con_'.$ctype.'_props', 'vals');
    }	
	
	public function writeFromForm($item, $table){

		$this->filterEqual('ctype', $item['ctype']);
		$this->filterEqual('title', $item['title']);
		$this->filterEqual('lang', $item['lang']);		
		$id = $this->getFieldFiltered('languages_'.$table, 'id');
		
		if($id) $this->update($table, $id, $item); else $this->insert('languages_'.$table, $item);
    }
	
	public function write($item, $type){

		if($type != 'ctypes') $this->filterEqual('ctype', $item['ctype']);
		$this->filterEqual('item_id', $item['item_id']);
		$this->filterEqual('lang', $item['lang']);		
		$id = $this->getFieldFiltered('languages_'.$type, 'id');
		
		if($id) $this->update('languages_'.$type, $id, $item); else $this->insert('languages_'.$type, $item);
    }
	
	public function createGeoTables($langs, $config_lang){
		foreach($langs as $lang){
			if($lang != $config_lang){
				$config = cmsConfig::getInstance();	
				$sql = "CREATE TABLE IF NOT EXISTS `{#}geo_cities_".$lang."` LIKE `{#}geo_cities`";				
				$result = $this->db->query($sql);
				$sql = "CREATE TABLE IF NOT EXISTS `{#}geo_countries_".$lang."` LIKE `{#}geo_countries`";				
				$result = $this->db->query($sql);
				$sql = "CREATE TABLE IF NOT EXISTS `{#}geo_regions_".$lang."` LIKE `{#}geo_regions`";				
				$result = $this->db->query($sql);								
			}
		}
    }	
	
	public function makeTable($ctype){
        	$config = cmsConfig::getInstance();	
			$sql = "CREATE TABLE IF NOT EXISTS `{#}languages_con_".$ctype."` (
					  `id` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
					  `item_id` int(11) UNSIGNED DEFAULT NULL,				  
					  `ctype` varchar(32) NOT NULL DEFAULT '',
					  `lang` varchar(5) NOT NULL DEFAULT '',
					  PRIMARY KEY (`id`)
					  ) ENGINE=".$config->db_engine." DEFAULT CHARSET=utf8";
				
			$result = $this->db->query($sql);	
    }
	
		public function makePropsTable($ctype){
        	$config = cmsConfig::getInstance();	
			$sql = "CREATE TABLE IF NOT EXISTS `{#}languages_con_".$ctype."` (
					  `id` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
					  `item_id` int(11) UNSIGNED DEFAULT NULL,				  
					  `ctype` varchar(32) NOT NULL DEFAULT '',
					  `lang` varchar(5) NOT NULL DEFAULT '',
					  `title` varchar(250) NOT NULL DEFAULT '',
					  `fieldset` varchar(128) NOT NULL DEFAULT '',					  
					  `vals` text NOT NULL DEFAULT '',					  					  
					  PRIMARY KEY (`id`)
					  ) ENGINE=".$config->db_engine." DEFAULT CHARSET=utf8";
				
			$result = $this->db->query($sql);	
    }		
	
	
	public function getItems($type, $ctype = false, $page = false, $perpage = false){

		$items = array();
		$model = cmsCore::getModel('content');
		$langs = cmsCore::getLanguages();
		if($ctype && $type == 'items') $this->makeTable($ctype);
								
        switch($type) {	
		
			# содержимое полей		
			case 'items':
				$model->limitPage($page, $perpage);			
				$articles = $model->get('con_'.$ctype);
				
				# находим все поля типа контента
				$fields = $model->getContentFields($ctype);
				
				# добавляем переведенные элементы
				
				foreach($articles as $key=>$article){
					foreach($langs as $lang){
						$translate = $this->getTranslateItem($ctype, $key, $lang);
						if($translate){
							$articles[$key]['translate'][$lang] = 1;
							foreach($translate as $f=>$v){
								unset($v['id']);
								unset($v['item_id']);							
								unset($v['ctype']);							
								unset($v['lang']);							
								foreach($v as $k=>$i){
								$articles[$key][$k.'_'.$lang] = $i;
								}
							}
						}
						
					}
				}

				
				
				
				/*foreach($fields as $key=>$field){

					$item = array(
						'item_id' => $field['id'], 
						'ctype' => $ctype,
						'type' => $field['type'], 
						'ftype' => $field['handler']->var_type
					);
						
					$langs = cmsCore::getLanguages();
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTitle($ctype, $field['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $field['title'];
					}						
					array_push($items, $item);	
				}*/
				$articles['all_fields'] = $fields;
				return $articles;
				break;
				
				
			
			# категории
			case 'cats':
				$cats = $this->get('con_'.$ctype.'_cats');
				foreach($cats as $key=>$cat){

					$item = array(
						'item_id' => $cat['id'], 
						'ctype' => $ctype,
						'type' => $ctype, 
						'ftype' => ''
					);
						

					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTitle($ctype, $cat['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $cat['title'];
					}						
					if($cat['parent_id'] != 0) array_push($items, $item);	
				}

				return $items;
				break;
				
				
				
			# поля
			case 'fields':
				$fields = $model->getContentFields($ctype);
				# заголовки полей
				foreach($fields as $key=>$field){

					$item = array(
						'item_id' => $field['name'], 
						'ctype' => $ctype,
						'type' => $field['type'], 
						'ftype' => $field['handler']->var_type
					);
					$item['title'] = $field['title'];

					foreach($langs as $lang) {

						$ff = $this->getTranslateField($ctype, $field['name'], $lang);
						if($ff){						
							foreach($ff as $fv) $item['title_'.$lang] = $fv['title'];
							if(!$item['title_'.$lang]) $item['title_'.$lang] = $field['title'];
						}
					}						
					array_push($items, $item);	
				}
				# подсказки к полям
				foreach($fields as $key=>$field){
						$item = array(
							'item_id' => $field['name'].'_hint', 
							'ctype' => $ctype,
							'type' => 'hint', 
							'ftype' => $field['handler']->var_type
						);
						$item['title'] = $field['hint'];
	
						foreach($langs as $lang) {
							$this->filterEqual('item_id', $field['name'].'_hint')->filterEqual('ctype', $ctype)->filterEqual('lang', $lang);
							$ff = $this->getFieldFiltered('languages_fields', 'title');
							if($ff)$item['title_'.$lang] = $ff;
						}						
						array_push($items, $item);	
				}					
				# единицы измерения
				foreach($fields as $key=>$field){

						if(isset($field['options']['units'])){
							$item = array(
								'item_id' => $field['name'].'_units', 
								'ctype' => $ctype,
								'type' => 'units', 
								'ftype' => $field['handler']->var_type
							);
							$item['title'] = $field['options']['units'];
		
							foreach($langs as $lang) {
								$this->filterEqual('item_id', $field['name'].'_units')->filterEqual('ctype', $ctype)->filterEqual('lang', $lang);
								$ff = $this->getFieldFiltered('languages_fields', 'title');
								if($ff)$item['title_'.$lang] = $ff;
							}						
							array_push($items, $item);	
						}
				}					
				# заголовки fielset
				$fieldsets = array();
				foreach($fields as $key=>$field){
					
						if(isset($field['fieldset'])){
							
							$item = array(
								'item_id' => $field['name'].'_fieldset', 
								'ctype' => $ctype,
								'type' => $field['fieldset'], 
								'ftype' => $field['handler']->var_type
							);
							$item['title'] = $field['fieldset'];
		
							foreach($langs as $lang) {
								$this->filterEqual('item_id', $field['name'].'_fieldset')->filterEqual('ctype', $ctype)->filterEqual('lang', $lang);
								$ff = $this->getFieldFiltered('languages_fields', 'title');
								if($ff)$item['title_'.$lang] = $ff;
							}						
							array_push($fieldsets, $item);	
						}
						
				}
				
				foreach($fieldsets as $item)  {
									
					if (!$this->searchArray($items, $item['title'])) array_push($items, $item);
				}
				
								
				
				# значения списков

				$fieldsets = array();
				foreach($fields as $key=>$field){
					
						if(isset($field['values'])){
							$item = array(
								'item_id' => $field['name'].'_values', 
								'ctype' => $ctype,
								'type' => $field['fieldset'], 
								'ftype' => $field['handler']->var_type
							);
							$item['title'] = $field['values'];
		
							foreach($langs as $lang) {
								$this->filterEqual('item_id', $field['name'].'_values')->filterEqual('ctype', $ctype)->filterEqual('lang', $lang);
								$ff = $this->getFieldFiltered('languages_fields', 'title');
								if($ff)$item['title_'.$lang] = $ff;
							}						
							array_push($fieldsets, $item);	
						}
						
				}

				foreach($fieldsets as $item)  {
									
					if (!$this->searchArray($items, $item['title'])) array_push($items, $item);
				}
	
				
				return $items;
				break;
				
								
			
			# наборы	
			case 'datasets':
				$ctype_id = $model->getContentTypeByName($ctype);
				$ctype_id = $ctype_id['id'];
				$datasets = $model->orderBy('title')->getContentDatasets($ctype_id);

				if($datasets)				
				foreach($datasets as $key=>$dataset){
					$dataset_ctype = $model->getContentType($dataset['ctype_id']);
					$item = array(
						'item_id' => $dataset['id'], 
						'ctype' => $dataset_ctype['name'], 
						'type' => $dataset_ctype['name'], 
						'ftype' => ''
					);
						
					//$langs = cmsCore::getLanguages();
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTitle($ctype, $dataset['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $dataset['title'];
					}						
					array_push($items, $item);	
				}
				return $items;
				break;
				
				
				
			# личные папки
			case 'folders':
				$ctype_id = $model->getContentTypeByName($ctype);
				$ctype_id = $ctype_id['id'];
				$folders = $this->filterEqual('ctype_id', $ctype_id)->get('content_folders');
								
				if($folders)
				foreach($folders as $key=>$folder){

					$item = array(
						'item_id' => $folder['id'], 
						'ctype' => $ctype,
						'type' => $ctype, 
						'ftype' => ''
					);
						
					//$langs = cmsCore::getLanguages();
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTitle($ctype, $folder['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $folder['title'];
					}						
					array_push($items, $item);	
				}

				return $items;
				break;
								
			case 'menu':
			
			$menu_items = $this->selectOnly('i.id, i.name, i.title')->get('menu');
			
			if($menu_items)
			foreach($menu_items as $key=>$menu_item){
					$item = array(
						'item_id' => $menu_item['id'], 
						'ctype' => $menu_item['name'],
						'type' => $ctype, 
						'ftype' => ''
					);
						
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateMenu($ctype, $menu_item['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $menu_item['title'];
					}						
					array_push($items, $item);	
				}

				return $items;
				break;
	
			case 'menu_items':

			$menu_items = $this->filterEqual('menu_id', $ctype)->selectOnly('i.id, i.menu_id, i.title')->get('menu_items');	
			
			if($menu_items)
			foreach($menu_items as $key=>$menu_item){
				//if($type == 'menu') $menu_item['menu_id'] = $menu_item['name'];
					$item = array(
						'item_id' => $menu_item['id'], 
						'ctype' => $menu_item['menu_id'],
						'type' => $ctype, 
						'ftype' => ''
					);
						
					//$langs = cmsCore::getLanguages();
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTitle($ctype, $menu_item['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $menu_item['title'];
					}						
					array_push($items, $item);	
				}

				return $items;
				break;
	
				
			case 'widgets':
				$widgets = $this->selectOnly('i.id, i.widget_id, i.title, i.links')->get('widgets_bind');
			
				if($widgets)
				foreach($widgets as $key=>$widget){
					$item = array(
						'item_id' => $widget['id'], 
						'ctype' => $widget['widget_id'],
						'type' => $ctype, 
						'links' => $widget['links']
					);
						
					//$langs = cmsCore::getLanguages();
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateMenu($ctype, $widget['id'], $lang, $type);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $widget['title'];
						$item['links_'.$lang] = $this->getTranslateMenuLinks($ctype, $widget['id'], $lang, $type);
						if(!$item['links_'.$lang]) $item['links_'.$lang] = $widget['links'];						
					}						
					array_push($items, $item);	
				}	
			
				return $items;
				break;
				
				
				case 'ctypes':
				$ctypes = $this->selectOnly('i.id, i.name, i.title, i.description, i.labels, i.seo_title, i.seo_keys, i.seo_desc')->get('content_types');

				if($ctypes)
				foreach($ctypes as $key=>$ct){
					$item = $ct;				
					$item['item_id'] = $ct['id'];
					$item['labels'] = cmsModel::yamlToArray($item['labels']);
					foreach($langs as $lang) {

						$translate = $this->filterEqual('item_id', $key)->filterEqual('lang', $lang)->get('languages_ctypes');	
						if($translate)												
						foreach($translate as $val) {
							$item['title_'.$lang] = $val['title'];
							if(!$item['title_'.$lang]) $item['title_'.$lang] = $ct['title'];	
							$item['description_'.$lang] = $val['description'];
							if(!$item['description_'.$lang]) $item['description_'.$lang] = $ct['description'];							
							$item['labels_'.$lang] = $val['labels'];
							if(!$item['labels_'.$lang]) $item['labels_'.$lang] = $ct['labels'];	
							$item['seo_keys_'.$lang] = $val['seo_keys'];
							if(!$item['seo_keys_'.$lang]) $item['seo_keys_'.$lang] = $ct['seo_keys'];							
							$item['seo_title_'.$lang] = $val['seo_title'];
							if(!$item['seo_title_'.$lang]) $item['seo_title_'.$lang] = $ct['seo_title'];	
							$item['seo_desc_'.$lang] = $val['seo_desc'];
							if(!$item['seo_desc_'.$lang]) $item['seo_desc_'.$lang] = $ct['seo_desc'];
							$item['labels_'.$lang] = cmsModel::yamlToArray($val['labels']);
							if(!$item['labels_'.$lang]) $item['labels_'.$lang] = $item['labels'];							
						}

					}
					
											
					array_push($items, $item);	
				}	
			
			
				return $items;
				break;
			
				
			# теги
			case 'tags':
				$this->limitPage($page, $perpage);
				$tags = $this->selectOnly('i.id, i.tag')->get('tags');
			
				if($tags)
				foreach($tags as $key=>$tag){
					$item = array(
						'item_id' => $tag['id'], 
						'ctype' => $tag['tag']
					);

					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslateTag($lang, $tag['id']);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $tag['tag'];						
					}						
					array_push($items, $item);	
				}	
				
				return $items;
				break;	
				
							
			# свойства
			case 'props':
				$this->makePropsTable($ctype.'_props');
				$props = $this->selectOnly('i.id, i.ctype_id, i.title, i.fieldset, i.values')->get('con_'.$ctype.'_props');

				if($props)
				foreach($props as $key=>$prop){
					$item = array(
						'item_id' => $prop['id'],
						'title' => $prop['title'],						 
						'ctype' => $ctype,
						'fieldset' => $prop['fieldset'],
						'values' => $prop['values']
					);

					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslatePropTitle($ctype, $lang, $prop['id']);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $prop['title'];
						$item['fieldset_'.$lang] = $this->getTranslatePropFieldset($ctype, $lang, $prop['id']);
						if(!$item['fieldset_'.$lang]) $item['fieldset_'.$lang] = $prop['fieldset'];						
						$item['values_'.$lang] = $this->getTranslatePropVals($ctype, $lang, $prop['id']);
						if(!$item['values_'.$lang]) $item['values_'.$lang] = $prop['values'];						
												
					}						
					array_push($items, $item);	
				}	
			
				# единицы измерения
				$model = cmsCore::getModel('content');
				$props = $model->getContentProps($ctype);
			if($props)
				foreach($props as $key=>$prop){
					if(isset($prop['options']['units']) && !empty($prop['options']['units'])){
						$item = array(
							'item_id' => $prop['id']+1000,
							'title' => $prop['title'],						 
							'ctype' => 'units',
							'fieldset' => $prop['title'],
							'values' => $prop['options']['units']
						);
	
						foreach($langs as $lang) {
						$item['title_'.$lang] = $this->getTranslatePropTitle($ctype, $lang, $prop['id']+1000);
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $prop['title'];
						$item['fieldset_'.$lang] = $this->getTranslatePropFieldset($ctype, $lang, $prop['id']+1000);
						if(!$item['fieldset_'.$lang]) $item['fieldset_'.$lang] = $prop['fieldset'];						
						$item['values_'.$lang] = $this->getTranslatePropVals($ctype, $lang, $prop['id']+1000);
						if(!$item['values_'.$lang]) $item['values_'.$lang] = $prop['values'];					
													
						}						
						array_push($items, $item);
					}
				}	
			
				return $items;
				break;
				
			# группы
			case 'groups':
				$groups = $this->selectOnly('i.id, i.title, i.description')->get('groups');

				if($groups)
				foreach($groups as $key=>$group){
					$item = array(
						'item_id' => $group['id'],
						'title' => $group['title'],						 
						'ctype' => $ctype,
						'description' => $group['description'],
					);

					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $group['id'])->filterEqual('lang', $lang)->filterEqual('ctype', 'groups')->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $group['title'];
						$item['description_'.$lang] =  $this->filterEqual('item_id', $group['id'])->filterEqual('lang', $lang)->filterEqual('ctype', 'description')->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['description_'.$lang]) $item['description_'.$lang] = $group['description'];												
												
					}	/**/					
					array_push($items, $item);	
				}	
			
			
			
				return $items;
				break;
				
				
			# активность
			case 'activity':
				$acts = $this->selectOnly('i.id, i.title, i.description')->get('activity_types');

				if($acts)
				foreach($acts as $key=>$act){
					$item = array(
						'item_id' => $act['id'],
						'title' => $act['title'],						 
						'ctype' => $ctype,
						'description' => $act['description'],
					);

					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $act['id'])->filterEqual('lang', $lang)->filterEqual('ctype', 'activity')->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $act['title'];
						$item['description_'.$lang] =  $this->filterEqual('item_id', $act['id'])->filterEqual('lang', $lang)->filterEqual('ctype', 'description')->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['description_'.$lang]) $item['description_'.$lang] = $act['description'];												
												
					}	/**/					
					array_push($items, $item);	
				}	
			
			
			
				return $items;
				break;
				
				
			# поля профиля
			case 'userfields':
				$userfields = $this->selectOnly('i.id, i.name, i.title, i.hint, i.fieldset')->get('users_fields');
				$hints = array();
				$sets = array();

				if($userfields)
				foreach($userfields as $key=>$userfield){
					$item = array(
						'item_id' => $userfield['id'],
						'title' => $userfield['title'],						 
						'ctype' => $userfield['name'],
					);
					
					if($userfield['hint']) array_push($hints, $userfield);
					if($userfield['fieldset']) array_push($sets, $userfield['fieldset']);
					
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $userfield['id'])->filterEqual('lang', $lang)->filterEqual('ctype', $userfield['name'])->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $userfield['title'];
											
												
					}						
					array_push($items, $item);	
				}	


				foreach($hints as $key=>$field){
					$item = array(
						'item_id' => $field['id'].'_hint',
						'title' => $field['hint'],						 
						'ctype' => 'hint',
						'type' => $field['name'].'_hint',
					);
					
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $field['id'])->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $field['hint'];							
					}
					array_push($items, $item);
				}
				
				$sets = array_unique($sets);
				foreach($sets as $key=>$field){
					$item = array(
						'item_id' => $userfield['name'].'_'.$key,
						'title' => $field,						 
						'ctype' => $field,
						'type' => $userfield['name'].'_'.$key,
					);
							$patt = str_replace(' ', '_', $field);			
					foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('ctype', $patt)->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $field;							
					}
					array_push($items, $item);				
				}	

							
				return $items;
				break;
				
				
			# вкладки профилей
			case 'usertabs':
				$usertabs = $this->selectOnly('i.id, i.title, i.name')->get('users_tabs');

				if($usertabs)
				foreach($usertabs as $key=>$tab){
					$item = array(
						'item_id' => $tab['id'],
						'title' => $tab['title'],						 
						'ctype' => $tab['title'],
					);

				foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $tab['id'])->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $tab['title'];																					
					}				
					array_push($items, $item);	
				}	
			
			
			
				return $items;
				break;
				
				
			# группы пользователей
			case 'usergroups':
				$usergroups = $this->selectOnly('i.id, i.title, i.name')->get('users_groups');

				if($usergroups)
				foreach($usergroups as $key=>$group){
					$item = array(
						'item_id' => $group['id'],
						'title' => $group['title'],						 
						'ctype' => $group['name'],
					);

				foreach($langs as $lang) {
						$item['title_'.$lang] = $this->filterEqual('item_id', $group['id'])->filterEqual('lang', $lang)->getFieldFiltered('languages_'.$ctype, 'title');
						if(!$item['title_'.$lang]) $item['title_'.$lang] = $group['title'];																					
					}				
					array_push($items, $item);	
				}	

				return $items;
				break;	
								
				
			default: return false;
		}
    }	
	
  	public function writeTags($tags, $langs, $config_lang){
		
		foreach($tags[$config_lang] as $key=>$tag){
			foreach($langs as $lang){
				if($lang != $config_lang){		
					$is_exists_original = $this->filterEqual('ctype', trim($tags[$config_lang][$key]))->filterEqual('lang', $lang)->get('languages_tags');	
					if(!$is_exists_original){
						$item = array('ctype' => strtolower(trim($tags[$config_lang][$key])), 'lang' => $lang, 'title' => strtolower(trim($tags[$lang][$key])));
						$this->insert('languages_tags', $item);
					}
				
				}
			}
		}
    }		
	
  
  	public function getContentField($ctype, $item_id, $lang){
        return $this->filterEqual('ctype', $ctype)->filterEqual('item_id', $item_id)->filterEqual('lang', $lang)->get('languages_con_'.$ctype);
    }	
  
  	public function TranslateItem($text, $lang, $key){ 
 		$query = http_build_query(array( 'key' => $key, 'text' => $text, 'lang' => $lang,)); 
		$response = file_get_contents('https://translate.yandex.net/api/v1.5/tr.json/translate?'.$query); 
		$translate = json_decode($response, true); 
		$translate = $translate['text'][0];
 
 		return $translate;
	}

  	public function Translate($text, $lang, $key){       	
		$parameters['text'] = $text;
		$parameters['lang'] = $lang;
		$parameters['key'] = $key;
		$parameters['format'] = 'html';
		
		$handler = curl_init();		
        curl_setopt($handler, CURLOPT_URL, static::BASE_URL . 'translate');
        curl_setopt($handler, CURLOPT_POST, true);
        curl_setopt($handler, CURLOPT_POSTFIELDS, http_build_query($parameters));
        curl_setopt($handler, CURLOPT_SSL_VERIFYHOST,0);			
    	curl_setopt($handler, CURLOPT_SSL_VERIFYPEER,0);
        $remoteResult = curl_exec($handler);
        if ($remoteResult === false) {
            throw new Exception(curl_error($handler), curl_errno($handler));
        }

        $result = json_decode($remoteResult, true);
        if (!$result) {
            $errorMessage = self::MESSAGE_UNKNOWN_ERROR;
            if (version_compare(PHP_VERSION, '5.3', '>=')) {
                if (json_last_error() !== JSON_ERROR_NONE) {
                    if (version_compare(PHP_VERSION, '5.5', '>=')) {
                        $errorMessage = json_last_error_msg();
                    } else {
                        $errorMessage = self::MESSAGE_JSON_ERROR;
                    }
                }
            }
            throw new Exception(sprintf('%s: %s', self::MESSAGE_INVALID_RESPONSE, $errorMessage));
        } elseif (isset($result['code']) && $result['code'] > 200) {
            throw new Exception($result['message'], $result['code']);
        }
		

        return $result;
    }  
  
  
  
  	public function processForm($ctype, $value, $val, $field, $system_lang, $data, $allow, $options, $controller){

		/*$url = explode('/', $controller->cms_core->uri);
		if($url[1] == 'edit') $action = 'edit'; else $action == 'add'; 
		$table = 'languages_con_'.$url[0];
		$exists_fields = $this->db->getTableFields($table);

		if(in_array($field->id, $exists_fields)){
			print_r($url);
			$value = $this->filterEqual('ctype', $ctype['name'])->getFieldFiltered($table, $field->id);
			
		}*/

		$content_model = cmsCore::getModel('content');	
		$langs = cmsCore::getLanguages();
		$config = cmsConfig::getInstance();
		$name = $field->name;
		
		// переводим заголовок поля
		$this->filterEqual('item_id', $field->name)->filterEqual('lang', $system_lang);
		$ftitle = $this->filterEqual('ctype', $ctype['name'])->getFieldFiltered('languages_fields', 'title');
		if($ftitle) $field->setOptions(array('title' => $ftitle)); else $ftitle = $field->title;
		
		// переводим подсказку к полю
		$this->filterEqual('item_id', $field->name.'_hint')->filterEqual('lang', $system_lang);
		$hint = $this->filterEqual('ctype', $ctype['name'])->getFieldFiltered('languages_fields', 'title');
		if($hint) $field->setOptions(array('hint' => $hint));
		
		# перевод единиц измерения		
		if(isset($field->options['units'])){
			$this->filterEqual('ctype', $ctype['name'])->filterEqual('lang', $system_lang)->filterEqual('item_id', $field->id.'_units');
			$units = $this->getFieldFiltered('languages_fields', 'title');		
		}
		if(!$ftitle) $ftitle = $field->title;
		if(!isset($units)) $units = '';
		$field->setOptions(array('title' => $ftitle, 'units' => $units));
		# перевод единиц измерения
		
		# перевод поля "Показывать на сайте"
		if($field->id == 'is_pub'){
			$labold = $this->filterEqual('id', $ctype['id'])->getFieldFiltered('content_types', 'labels');
			$labold = $this->yamlToArray($labold);
			$labels = $this->filterEqual('item_id', $ctype['id'])->filterEqual('lang', $system_lang)->getFieldFiltered('languages_ctypes', 'labels');
			$labels = $this->yamlToArray($labels);
			if($labels) $field->setOptions(array('title' => str_replace($labold['create'], $labels['create'], $field->title)));
			$val = $field->getInput($value); 
		}
		# перевод поля "Показывать на сайте"
		
		# перевод текстовых полей
		if($allow && isset($field->type) && ($field->type == 'html' || $field->type == 'text')){
			$val = '';
			$buttons = '';
			$sections = '<div id="'.$name.'_container" class="translate_fields">';	

			foreach($langs as $lang) {
				if($lang != $config->language) $element = $name.'_'.$lang; else $element = $name;
				$field->setOptions(array('title' => $ftitle.' '.strtoupper($lang), 'element_name' => $element)); 				
				if($lang == $system_lang) {
					$ch = 'class="active"'; 
					$cha = 'active';
				}else {
					$ch = '';
					$cha = '';
				}

				if(isset($field->item['id'])) $item_id = $field->item['id']; else $item_id = -1;
				if(!isset($data[$element])) $text_tr = $this->filterEqual('item_id', $item_id)->filterEqual('lang', $lang)->getFieldFiltered('languages_con_'.$ctype['name'], $field->id);
				
				if(isset($data[$element])) $text_tr = $data[$element];
				if($text_tr) $text = $field->getInput($text_tr); else $text = $field->getInput($value);		

				$buttons .='<li '.$ch.'><img src="/templates/default/controllers/languages/images/flags/flag_'.$lang.'.png"></li>';				
				$sections .= '<div class="notebooks__content '.$cha.'" id="'.$element.'_text">
				<a lang="'.$lang.'" class="lang_link" onclick="ya_get(this)" href="javascript:void(0)" id="'.$lang.'" mark="'.$element.'">
				<img style="float:right;" src="/templates/default/controllers/languages/images/flags/flag_'.$lang.'.png" title="'.LANG_LANGS_AUTOTRANSLATE.'">
				</a>'.$text.'</div>';

			}
			$val .= '<div class="notebooks"><ul class="notebooks__caption">'.$buttons.'</ul>'.$sections.'</div>';
			$sections = '</div>';
		} else { 
		
			$val = $field->getInput($value); 
		
		}
		# перевод текстовых полей


		# перевод остальных полей
		if(isset($field->type) && ($field->type != 'caption' && $field->type != 'text' && $field->type != 'html')){	

			$val = $field->getInput($value);	
		}
		# перевод остальных полей
		
		# перевод списков
		if(isset($field->type) && $field->type == 'list'){ 
			$this->filterEqual('ctype', $ctype['name'])->filterEqual('lang', $system_lang)->filterEqual('item_id', $field->id.'_values');
			$is_t = $this->getFieldFiltered('languages_fields', 'title');
			$is_t = explode("\r\n", $is_t);
			$fi_t = explode("\r\n", $field->values);
			foreach($is_t as $k=>$t){
				if(strstr($t, '|')) {
					$is_t[$k] = explode("|", $is_t[$k]);
					$fi_t[$k] = explode("|", $fi_t[$k]);
					$val = str_replace(trim($fi_t[$k][1]), trim($is_t[$k][1]),$val);							
				} else {
					$val = str_replace(trim($fi_t[$k]), trim($is_t[$k]),$val);
				}

			}
						
		}
		# перевод списков	
		
		# перевод строковых полей
		if($allow && (isset($field->type) && $field->type == 'caption')){
			$temp_val = array();
			$temp_val[0] = '';
			$val = '<div id="'.$name.'_container" class="translate_fields">';
			$counter = 1;

			foreach($langs as $lang) {
				if($lang != $config->language) $element = $name.'_'.$lang; else $element = $name;
			

				$field->setOptions(array('title' => $ftitle.' '.strtoupper($lang), 'element_name' => $element));

				if(isset($field->item['id'])) $item_id = $field->item['id']; else $item_id = -1;
				if($lang == $config->language){
					//if(!isset($data[$element])) 
					$text_tr = $this->filterEqual('id', $item_id)->getFieldFiltered('con_'.$ctype['name'], $field->id);
				} else {
					//if(!isset($data[$element])) 
					$text_tr = $this->filterEqual('item_id', $item_id)->filterEqual('lang', $lang)->getFieldFiltered('languages_con_'.$ctype['name'], $field->id);
				}
					
				//if(isset($data[$element])) $text_tr = $data[$element];

				
				if(isset($text_tr) && $text_tr) $text = $field->getInput($text_tr); else $text = $field->getInput($value);	

				$html = $text.'<div class="notebooks" id="'.$element.'_text">
				<a lang="'.$lang.'" onclick="ya_get(this)" href="javascript:void(0)" id="'.$lang.'" mark="'.$element.'" style="float:right; margin-top:-25px;">
				<img src="/templates/default/controllers/languages/images/flags/flag_'.$lang.'.png" title="'.LANG_LANGS_AUTOTRANSLATE.'"></a></div>';

				if($lang != $system_lang){ $temp_val[$counter] = $html; } else { $temp_val[0] = $html; }
				$counter++;
					
			}
			
			foreach($temp_val as $tv) $val .= $tv;
			$val .= '</div>';
			
		}
		# перевод строковых полей	



		#данное поле является свойством
		if(strstr($field->name, 'props:')){ 
			$dat = explode(':', $field->name);

			$this->filterEqual('ctype', $ctype['name'])->filterEqual('lang', $system_lang)->filterEqual('item_id', $dat[1]);
			$ftitle = $this->getFieldFiltered('languages_con_' .$ctype['name'].'_props', 'title');
			if(isset($field->units)){
				$this->filterEqual('ctype', $ctype['name'])->filterEqual('lang', $system_lang)->filterEqual('item_id', $dat[1] + 1000);
				$units = $this->getFieldFiltered('languages_con_' .$ctype['name'].'_props', 'vals');
			}
			if(!$ftitle) $ftitle = $field->title;
			if(!isset($units)) $units = '';
			$field->setOptions(array('title' => $ftitle, 'units' => $units));			 
			$val = $field->getInput($value);
			
			if(isset($field->items) && $field->items){	
				$this->filterEqual('ctype', $ctype['name'])->filterEqual('lang', $system_lang)->filterEqual('item_id', $dat[1]);
				$vals = $this->getFieldFiltered('languages_con_' . $ctype['name'].'_props', 'vals');		
				$vals = explode("\r\n", $vals);

				foreach($vals as $k=>$t){
					$val = str_replace(trim($field->items[$k+1]), trim($vals[$k]),$val);
				}
			}
		}
		#данное поле является свойством


		# перевод списка категорий
		if($field->name == 'category_id'){
			$cats = $content_model->get('con_'.$ctype['name'].'_cats');					
			if($cats)
			foreach($cats as $key=>$cat){
				$cat_title = $this->getCatTitle($ctype['name'], $cat['id'], $system_lang);	
				if($cat_title) {
					$val = str_replace($cat['title'].'</option>', $cat_title.'</option>', $val);
					$val = str_replace($cat['title'].'">', $cat_title.'">', $val);					
				}
			}	
		}
		# перевод списка категорий
		
				
		# перевод списка папок
		if($field->id == 'folder_id'){
			
			$folders = $content_model->get('content_folders');
			if($folders)
			foreach($folders as $key=>$folder){			
				$folder_title = $this->filterEqual('item_id', $folder['id'])->filterEqual('lang', $system_lang)->getFieldFiltered('languages_folders', 'title');
				if($folder_title) $val = str_replace($folder['title'], $folder_title, $val);
			}	
			
		}
		# перевод списка папок
		
		# перевод списка групп
		if($field->id == 'parent_id'){
			
			$groups = $content_model->get('groups');
			if($groups)
			foreach($groups as $key=>$group){			
				$group_title = $this->filterEqual('item_id', $group['id'])->filterEqual('lang', $system_lang)->getFieldFiltered('languages_groups', 'title');
				if($group_title) $val = str_replace($group['title'], $group_title, $val);
			}	
			
		}
		# перевод списка групп		
		
		
		
        return array($val, $field);
    }	  
  
 
  
  
  
    public function processStandartForm($value, $val, $field, $system_lang, $data){

		# перевод полей профиля
		$f_id = $this->filterEqual('name', $field->id)->getFieldFiltered('users_fields', 'id');				
		$f = $this->filterEqual('item_id', $f_id)->filterEqual('lang', $system_lang)->get('languages_userfields');
		if($f)
		foreach($f as $f_item){
			if($f_item['ctype'] == $field->id) $field->setOptions(array('title' => $f_item['title']));
			if($f_item['ctype'] == 'hint') $field->setOptions(array('hint' => $f_item['title']));
		}
		$val = $field->getInput($value);	
		
		# перевод полей профиля		
		
		# перевод списка категорий
		if($field->id == 'parent_id'){
			$content_model = cmsCore::getModel('content');
			$cats = $content_model->get('con_'.$field->item['ctype_name'].'_cats');
		
			if($cats)
			foreach($cats as $key=>$cat){
				$cat_title = $this->getCatTitle($field->item['ctype_name'], $cat['id'], $system_lang);	
				if($cat_title) $val = str_replace($cat['title'].'</option>', $cat_title.'</option>', $val);
			}	
		}
		# перевод списка категорий	
		
		# перевод поля доступ		
		if($field->id == 'allow_add'){
			$groups = $this->get('users_groups');				
			$f_groups = $this->get('languages_usergroups');
			$groups_data = $field->data;
			foreach($groups as $key=>$f_item){
				if(isset($groups_data['groups'][$key]) && $groups_data['groups'][$key] == $f_item['title']) $hhh = $f_item['id']; 
				if(isset($hhh))
					foreach($f_groups as $f_group){
						if($f_group['item_id'] == $hhh) $val = str_replace($f_item['title'], $f_group['title'], $val);
					}
			}
				
		}					
		# перевод поля доступ

		return array($val, $field);

	}
  
  
  

  
  
  	public function searchArray($array, $val){
		foreach ($array as $key => $value){
			if (array_search ($val, $value)){
                return true;
			}   
		}
	}
  
  
  
  

}